/* $Id: ata_identify.c,v 1.3 2004/07/14 11:56:42 harald Exp $ */
#include <stdarg.h>
#include <stdio.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>

#include <syslog.h>
#include <sysfs/libsysfs.h>
#include <linux/types.h>
#include <linux/hdreg.h>

static int debug;
static int hotplug_mode;
static int use_stderr;

void log_message (int level, const char *format, ...)
{
	va_list	args;

	if (!debug && level == LOG_DEBUG)
		return;

	va_start (args, format);
	if (!hotplug_mode || use_stderr) {
		vfprintf(stderr, format, args);
	} else {
		static int logging_init = 0;
		static unsigned char logname[32];
		if (!logging_init) {
			/*
			 * klibc does not have LOG_PID.
			 */
			snprintf(logname, 32, "ata_identify[%d]", getpid());
			openlog (logname, 0, LOG_DAEMON);
			logging_init = 1;
		}

		vsyslog(level, format, args);
	}
	va_end (args);
	return;
}


int main(int argc, char **argv)
{
	struct hd_driveid id;
	int fd, i, j, l, r;
	char unique[sizeof(id.model) + sizeof(id.serial_no) + 1];

	if (getenv("DEBUG"))
		debug++;

	if ((argc == 2) && (argv[1][0] != '-')) {
		hotplug_mode = 1;
		/* dprintf("hotplug assumed\n"); */
	}

	fd = open(argv[1], O_RDONLY | O_NONBLOCK);
	if (fd < 0) {
		perror(argv[0]);
		return 1;
	}

	if (!ioctl(fd, HDIO_GET_IDENTITY, &id)) {

		l = r = unique[sizeof(unique) - 1] = 0;

		for (j = sizeof(id.model); j >= 0; --j)
			if (id.model[j] && id.model[j] != ' ')
				break;
		for (i = 0; i < j; i++) {
			if (id.model[i] == ' ') {
				if (!l)
					continue;
				id.model[i] = '_';
			}
			if (!id.model[i])
				break;
			unique[r++] = id.model[i];
			l++;
		}
		l = 0;
		for (j = sizeof(id.serial_no); j >= 0; --j)
			if (id.serial_no[j] && id.serial_no[j] != ' ')
				break;
		if (id.serial_no[0] && id.serial_no[0] != ' ')
			unique[r++] = '_';
		for (i = 0; i < j; i++) {
			if (id.serial_no[i] == ' ') {
				if (!l)
					continue;
				id.serial_no[i] = '_';
			}
			if (!id.serial_no[i])
				break;
			unique[r++] = id.serial_no[i];
			l++;
		}
		unique[r] = '\0';
		printf("%s\n", unique);
	} else if (errno == -ENOMSG)
		fprintf(stderr, " no identification info available\n");
	else
		fprintf(stderr, " HDIO_GET_IDENTITY failed on %s\n", argv[1]);

	close(fd);
	return 0;
}
