#!/bin/sh
# $Id: udev.get_unique_drive_id.sh,v 1.3 2004/06/08 11:50:56 harald Exp $
#set -x

# this script should provide the disk id 

# example for all:
# for i in `find /sys/block -name dev` ;do DEVPATH="`echo $i | sed -e 's@^/sys\|/dev@@g'`" $0 ; done

PATH=/bin:/usr:/usr/sbin:/usr/bin:/sbin:/bin
SYSFS=/sys
PATH=/sbin:$PATH
export PATH
RESULT=1
TYPE=

. /etc/udev/udev.conf

pwd() {
        if [ -x /bin/pwd ]; then
                /bin/pwd;
        else
                pwd;
        fi
}


OPWD="`pwd`"
full_sysfs_class_path=
full_sysfs_device_path=
is_partition=false

get_scsi_type () {
    local type

    read type < $1/type
    case "$type" in
	0)
	    # Direct-access device
	    devtype=disk
	    ;;
	1)
	    # Sequential access device
	    devtype=tape
	    ;;
	2)
	    # Printer device
	    devtype=printer
	    ;;
	3)
	    # Processor device
	    devtype=scanner
	    ;;
	4)
	    # Write-once device
	    devtype=cdroms
	    ;;
	5)
	    # CD-ROM device
	    devtype=cdroms
	    ;;
	6)
	    # Scanner device
	    devtype=scanner
	    ;;
	7)
            # Optical memory device
	    devtype=disk
	    ;;
	8)
	    # Medium changer devices
	    devtype=generic
	    ;;
	9)
	    # Communications device
	    devtype=generic
	    ;;
	10|11)
	    # Graphic arts pre-press device
	    devtype=generic
	    ;;
	12)
	    # Storage array controller
	    devtype=generic
	    ;;
	13)
	    # Enclosure service device
	    devtype=generic
	    ;;
	14)
	    # Simplified direct-access device
	    devtype=generic
	    ;;
	15)
	    # Optical card reader/writer device
	    devtype=generic;
	    ;;
	*)
	    devtype=generic;
	    ;;
    esac

    echo $devtype
}

if [ -z "$DEVPATH" -a -z "$1" ] ; then
exit 1
fi
if [ -z "$DEVPATH" ] ; then
	case "$1" in
		$SYSFS/*)
		DEVPATH="${1#$SYSFS}"
		;;
		*)
		DEVPATH=$1
		;;
	esac
fi

if [ ! -d $SYSFS$DEVPATH ] ; then
exit 1
fi
if [ ! -f $SYSFS$DEVPATH/dev ] ; then
exit 1
fi


case "$DEVPATH" in
	/block/*)
		TYPE=block
		;;
	/class/*)
		TYPE="${DEVPATH#/class/}"
		TYPE="${TYPE%%/*}"
		;;
	*)
		exit 1
		;;
esac
		

#
##
#


case "$TYPE" in
	scsi_generic)
	cd $SYSFS$DEVPATH/device/block || exit 1
	DEVPATH="`pwd`"
	cd "$OPWD"
	DEVPATH="${DEVPATH#$SYSFS}"
	;;
esac

if [ -f $SYSFS$DEVPATH/range -a -L $SYSFS$DEVPATH/device ] ; then
	read major_minor < $SYSFS$DEVPATH/dev
	cd $SYSFS$DEVPATH/device
	fulldevpath="`pwd`"
	if [ -d block ]; then
	    cd block
	    fullblockpath="`pwd`"
	fi
	cd "$OPWD"
else
	if [ ! -L $SYSFS$DEVPATH/../device -a ! -f $SYSFS$DEVPATH/../range ] ; then
		exit 1
	fi
	read major_minor < $SYSFS$DEVPATH/../dev
	cd $SYSFS$DEVPATH/../device
	fulldevpath="`pwd`"
	if [ -d block ]; then
	    cd block
	    fullblockpath="`pwd`"
	fi
	cd "$OPWD"
	is_partition=true
fi

major=${major_minor%%:*}
minor=${major_minor##*:}
mynode="/$udev_root/diskid_tmpnode.$major.$minor.$$"
devtype="generic"
#
#
handle_block () {
case "$fulldevpath" in
	# USB devices have a serial number. if not, too bad
	*/usb*/*/host*/*)
		: skip usb: $fulldevpath
		return 1
		;;
	# ide-scsi is CD only
	*/ide*/*/host*/*)
		: skip ide-scsi: $fulldevpath
		return 1
		;;
	# scsi devices
	# firewire sbp2 has a GUID, but it may provide the serial of the disk
	*/host[0-9]*/[0-9]*:[0-9]*:[0-9]*:[0-9]*)
		# check for ieee1394 sbp2 
		# do not call scsi_id for firewire disks, it may hang
		if [ -f $fulldevpath/ieee1394_id ] ; then
			return 1
		fi
		: looking at scsi $fulldevpath
		disk_id="`scsi_id -p 0x80 -gs $DEVPATH`"
		: disk_id XXX${disk_id}XXX
		# Fallback to normal id if page 80 is not supported
		if [ ! $? ]; then
		    disk_id="`scsi_id -gs $DEVPATH`"
		fi
		disk_id="`echo $disk_id | sed -e 's@ \+@_@g'`"
		if [ -f $fulldevpath/type ] ; then
		    devtype="`get_scsi_type $fulldevpath`"
		fi
	;;
	# ide devices
	*/ide[0-9]/[0-9].[0-9]*|*/ide[0-9][0-9]/[0-9][0-9].[0-9]*)
		
		: looking at ide $fulldevpath
		rm -f $mynode
		mknod -m 600 $mynode b $major $minor
		disk_id="`ata_identify $mynode`"
		disk_id=${disk_id/\//_}
		rm -f $mynode
		if [ ! $? ]; then
		    disk_id=
		fi
		if [ -f /proc/ide/${fullblockpath##*/}/media ]; then
		    read media < /proc/ide/${fullblockpath##*/}/media
		    case "$media" in
			disk)
			    devtype=disk
			    ;;
			cdrom)
			    devtype=cdroms
			    ;;
			tape)
			    devtype=tape
			    ;;
			floppy)
			    devtype=disk
			    ;;
			*)
			    devtype=generic
			    ;;
		    esac
		fi
	;;
	*/css0/*)
		: looking at dasd $fulldevpath
		rm -f $mynode
		mknod -m 600 $mynode b $major $minor
	        disk_id="`/sbin/dasdview -j -f $mynode`"
		rm -f $mynode
		if [ ! $? ]; then
		    disk_id=
		else
		    devtype=disk
		fi
	;;
	*)
		: $fulldevpath not handled
		return 1
	;;
esac
# done
return 0
}

case "$TYPE" in
	block)
	if ! handle_block; then
	    : not handled
	    exit 1
	fi

	if [ -z "$disk_id" -o "$devtype" = "generic" ]; then
	    : not handled
	    exit 1
	fi

	# look for a partition number
	if [ "$is_partition" = "true" ] ; then
		dp="`echo $DEVPATH | sed -e 's@^.*[^0-9]@@'`"
		case "$disk_id" in
			*[0-9])
			disk_id="${disk_id}p${dp}"
			;;
			*)
			disk_id="${disk_id}${dp}"
			;;
		esac
	fi
	if [ ! -z "$disk_id" ] ; then
		echo "$devtype/by-id/$disk_id"
	fi
	RESULT=0
	;;
	scsi_generic)
	d="`handle_block`"
	if [ ! -z "$d" ] ; then
		echo "${d}-generic"
		RESULT=0
	fi
	;;
	*)
	RESULT=1
	;;
esac
exit $RESULT
